"""
Modules:
    - internal:
        - description: Already Comes installed with your python interpreter.
    - external:
        - description: Needs to be installed using pip.
    - Project Specific
        - description: Modules made for this project.
        - contents:

Hierarchy:
    - exceptions.py:
        - contents:
            InvalidKeyword
            InvalidExportType
            NoneTypeVariable
            - OptError (class):
                - contents:
                    OPTIONERROR (const)
            - RestrictedActivity (class):
                - contents:
                    RESTRICTEDACTIVITY (const)
InvalidKeyword:
    - description: Exception for Invalid Keyword.

InvalidExportType:
    - description: Exception for Invalid Export Type.

NoneTypeVariable:
    - description: Exception for None Type Variable.

OptError:
    - description: Exception for Option Error.
    
    - OPTIONERROR (const):
        - description: contains default prefix text for OptError
        - value: 'Option Error:'

RestrictedActivity:
    - description: Exception for Restricted Activity.

    - RESTRICTEDACTIVITY (const):
        - description: Contains default prefix text for RestrictedActivity Exception.
        - value: 'This activity is not permitted:'            

InvalidExportType:
    - description: Exception for Invalid File type of import
    
    - INVALIDIMPORTTYPE (const):
        - description: contains default prefix text for InvalidExportType Exception.
"""

# Invalid Keyword Exception
class InvalidKeyword(Exception):
    pass

# Invalid Export Type Exception
class InvalidExportType(Exception):
    pass

# None Type Variable Exception
class NoneTypeVariable(Exception):
    pass

# Option Error Exception
class OptError(Exception):
    OPTIONERROR = "Option Error:"
    pass

# Restricted Activity Exception
class RestrictedActivity(Exception):
    RESTRICTEDACTIVITY = "This activity is not permitted:" # exception constant
    pass

class InvalidImportType(Exception):
    INVALIDIMPORTTYPE = "This File format is not yet supported:"
    pass


#!/usr/bin/env python3
"""
Modules:
    - internal:
        - description: Already Comes installed with your python interpreter.
        - contents:
            pathlib.Path (class)
            shutil.rmtree (variable)
            os.makedirs (function)
            os.system (function)
            os.path.abspath (function)
            os.path.join(function):
                - alias: 'jPath'
            os.path.exists (function):
                - alias: 'there'
            threading (module)
            sys (module)
            platform (module)
    - external:
        - description: Needs to be installed using pip.
        -contents:
            tkinter.* (all)
            tkinter.simpledialog (module)
            colorama.init (function)
            colorama.Fore (variable)
            optioner.options (class)
    - Project Specific
        - description: Modules made for this project.
        - contents:
            pasta_man.architectures.gui.pmanager (class)
            pasta_man.encryption.Encryption (class)
            pasta_man.exceptions.NoneTypeVariable (class)
            pasta_man.exception.OptError (class)
            pasta_man.utilities.helptext.helptext (class)

Hierarchy:
    - pasta_man.py
        - contents:
            locker (function)
            unlocker (function)
            checkmfile (function)
            main (function)

locker:
    - description: lock function to lock master password. This is called as a thread.

unlocker:
    - description: unlock function to unlock master password. This is called as a thread.

checkmfile:
    - description: check for master password file.

main:
    - descripton: main driver function.

Working:
    - catch and analyse arguments if any. If Found, execute it. Else Move on.
    - Check for master password
    - If found, decrypt it and call pmanager class. If not found, ask the user for master password, encrypt and save it and then call pmanager class.
"""

# version info
__version__ = "1.1.3"

# import internal modules
from pathlib import Path
from os.path import join as jPath, exists as there, dirname, abspath
from os import makedirs
from shutil import rmtree
import sys, threading

# import external modules
from tkinter import *
from tkinter import simpledialog
from colorama import init as color, Fore as f

# import project specific modules
from pasta_man.architectures.gui import pmanager
from pasta_man.architectures.terminal import Terminal
from pasta_man.utilities.encryption import Encryption
from pasta_man.utilities.Exceptions.exceptions import NoneTypeVariable
from pasta_man.utilities.Exceptions.TerminalExceptions import EmptyArgument

# locker function
def locker(masterpassword: str):
    """
    Function to lock the encryption using a master password.

    Args:
        masterpassword (str): The master password used for encryption.

    Returns:
        None
    """
    global encb
    enc = Encryption(masterpassword.encode('ascii'))
    encb = enc.lock()
    sys.exit(0)

def unlocker(encpassword: str):
    """
    Function to unlock encrypted data using a password.

    Args:
        encpassword (str): The password used to unlock the encrypted data.

    Returns:
        None
    """
    global dencb
    denc = Encryption(encpassword.encode('ascii'))
    dencb = denc.unlock()
    sys.exit(0)


def checkmfile(home = str(Path.home())) -> bytes:
    """
    Check if the master password is defined and retrieve it from a file if it exists.

    Args:
        home (str | optinal): The home directory path. Defaults to the user's home directory.

    Returns:
        bytes: The master password.

    Raises:
        NoneTypeVariable: If the master password is empty or None.

    """
    # --> find out if master password is defined -> .m file
    if not there(jPath(home, '.pastaman', '.m')):
        # ask dialog
        masterpassword = simpledialog.askstring("User Input", "Enter Master Password: ", show='-')
        # check if it is empty or non
        if masterpassword=='' or masterpassword==None:
            raise NoneTypeVariable('Aborted.')
        # encode it
        masterpassword = masterpassword.encode('ascii')
        enct = threading.Thread(target=locker, args=(masterpassword.decode('ascii'),))
        enct.start()
        enct.join()
        # -> store it
        with open(jPath(home, '.pastaman', '.m'), 'wb') as m:
            m.write(encb)
    else:
        # if m is present
        # -> read it
        with open(jPath(home, '.pastaman', '.m'), 'rb') as m:
            masterpassword = m.read() # this is encrypted
        
        # -> decrypt it using the Encryption class
        # denc = Encryption("pastaman".encode('ascii'), masterpassword)
        # denc.unlock()
        denct = threading.Thread(target=unlocker, args=(masterpassword.decode('ascii'),))
        denct.start()
        denct.join()
        
        # fetch it again
        masterpassword = dencb
    
    return masterpassword

def main():
    """
    The main function of the program.
    """
    # initialize colorama
    color()
    # -> home folder
    home = str(Path.home())
    # -> find out if .pastaman folder is there
    if not there(jPath(home, '.pastaman')):
        makedirs(jPath(home, '.pastaman'))
    
    masterpassword = checkmfile()
    
    # define global vars
    global encb, dencb
    
    
    try:
        terminal = Terminal(__version__, sys.argv[1:], masterpassword)
        # clear __pycache__
        # -> get directory
        install_directory = dirname(abspath(__file__))
        rmtree(jPath(install_directory, '__pycache__'))
        sys.exit(0)
    except EmptyArgument:
        pass
    
    # define main root gui window
    rootwindow = Tk()
    
    # create pass manager object.
    pManager = pmanager(rootwindow, masterpassword)
    pManager._makeinitscreen_()
    
    rootwindow.mainloop()

if __name__=="__main__":
    encb:bytes = ''.encode('ascii')
    dencb:bytes = ''.encode('ascii')
    try:
        sys.exit(main())
    except KeyboardInterrupt:
        print("\n"+f"{f.RED}KEYBOARD INTERRUPT{f.RESET}")
        sys.exit(1)


"""
Modules:
    - internal:
        - description: Already Comes installed with your python interpreter.
        - contents:
            platdorm.system (function)
            os.system (function)
            os.makedirs (function)
            os.chdir (function)
            os.path.exists (function)
            os.path.join (function)
            os.path.dirname (function)
            os.path.abspath (function)
            pathlib.Path (class)
            sys
            subprocess
    - external:
        - description: Needs to be installed using pip.
    - Project Specific
        - description: Modules made for this project.

Hierarchy:
    - Launcher.py
        - contents:
            checklogfile (function)
            makePasta (function)
            main (function)

checklogfile:
    - description: Check for log file, if not present, create it.

makePasta:
    - description: Creates a .bat file for windows which will be run to setup pasta-man for Windows (one-time)

main:
    - description: checks whether system is windows or others. If others, run pasta-man as an independent process, else run a vbs script in the background for windows to launch an independent pasta-man process.
            
Working:
    - Check system
    - if not windows, run pasta-man as an independent process (no setup required)
    - if windows, run first time setup, where an exe and vbs files are created. From next time, run exe.
"""

from platform import system as os
from os import system as run, makedirs, chdir
from os.path import exists as there, join as jPath, dirname, abspath, basename
from pathlib import Path
from wrapper_bar.wrapper import Wrapper
import sys
from colorama import init as color, Fore as f

def checklogfile():
    """## check for .log file, if not present, create it
    """
    if not there(jPath(str(Path.home()), '.pastaman')):
        makedirs(jPath(str(Path.home()), '.pastaman'))
    
    if not there(jPath(str(Path.home()), '.pastaman', '.log')):
        with open(jPath(str(Path.home()), '.pastaman', '.log'), 'w') as logfile:
            logfile.write("log file init")

def makePasta():
    batdat = """
@REM turn printing of commands off
@echo off

@REM install using pyinstaller.
pyinstaller --onefile --noconsole pasta_man.py > NUL

@REM After this delete .spec file and build folder tree.
del build /s /q
del pasta_man.spec

@REM copy dist/pasta_man.exe to %USERPROFILE%\.pastaman\pasta-man.exe
copy dist\pasta_man.exe %USERPROFILE%\.pastaman\pasta-man.exe

@REM delete dist
del dist /s /q

@REM create vbs script
cd %USERPROFILE%\.pastaman
echo Set pastaShell = WScript.CreateObject('WScript.Shell') > pasta-man.vbs
echo pastaShell.Run "%USERPROFILE%\.pastaman\pasta-man.exe", 0, False >> pasta-man.vbs

@REM copy it 
"""
    directory = dirname(dirname(abspath(__file__))) # pasta_man directory
    chdir(directory)
    print(f'{f.RED}(one-time-setup){f.RESET}')
    print('Operating System: Windows\nsetting up pasta-man...\nThis might take a while.')
    
    wrap = Wrapper()
    
    codes = [
        f"""subprocess.Popen(['pyinstaller', '--onefile', '--noconsole', 'pasta_man.py'], stderr=subprocess.DEVNULL, stdout=subprocess.DEVNULL).wait()""",
    ]
    
    dependencies = [
        """import subprocess"""
    ]
    
    wrap.pyShellWrapper(codes, dependencies, delay=0.2, width=70, label="Making pasta:", timer="ElapsedTime")
    
    codes = [
        """shutil.rmtree(join(pwd(), 'build'))""",
        """remove(join(pwd(), 'pasta_man.spec'))""",
        """shutil.copyfile(join(pwd(), 'dist', 'pasta_man.exe'), join(str(Path.home()), '.pastaman', 'pasta-man.exe'))"""
    ]
    
    dependencies = [
        """import shutil""",
        """from os.path import join""",
        """from os import getcwd as pwd, remove""",
        """from pathlib import Path"""
    ]
    
    wrap.pyShellWrapper(codes, dependencies, delay=0.03, width=62, label="Cleaning up:")
    
    codes = ["""
path = join(str(Path.home()), '.pastaman')
with open(join(path, 'pasta-man.vbs'), 'w') as vbs:
    vbs.write("Set pastashell = WScript.CreateObject(\\"WScript.Shell\\")\\n")
    vbs.write("pastashell.Run \\"%USERPROFILE%\\\\.pastaman\\\\pasta-man.exe\\", 0, False")
"""]
    
    dependencies = [
        """from pathlib import Path""",
        """from os.path import join"""
    ]
    
    wrap.pyShellWrapper(codes, dependencies, delay=0.01, width=60, label="Garnishing:")
    
    print(f"{f.LIGHTGREEN_EX}Serving...{f.RESET}")
def main():
    color()
    checklogfile()
    # check for arguments
    if len(sys.argv[1:])>0:
        sys.argv[0] += sys.argv[0]+'-launcher'
        command = basename(sys.argv[0])
        for i in range(1, len(sys.argv)):
            command += " " + sys.argv[i]
        
        run(command)
        # exit
        sys.exit(0)
    
    # if no arguments run these
    system = os()
    if system=='Darwin' or system=='Linux':
        run('nohup pasta-man-launcher > ~/.pastaman/.log &')
    elif system=='Windows':
        if not there(jPath(str(Path.home()), '.pastaman', 'pasta-man.vbs')):
            makePasta()
        chdir(jPath(str(Path.home()), '.pastaman'))
        run(f"start /B wscript.exe {jPath(str(Path.home()), '.pastaman', 'pasta-man.vbs')}")
    sys.exit(0)


"""
Modules:
    - internal:
        - description: Already Comes installed with your python interpreter.
        - contents:
            base64.urlsafe_b64encode (function)
    - external:
        - description: Needs to be installed using pip.
        -contents:
            cryptography.fernet.Fernet (class)
            cryptography.hazmat.primitives.hashes.SHA256 (class)
            cryptography.hazmat.primitives.kdf.pbkdf2.PBKDF2HMAC (class)
    - Project Specific
        - description: Modules made for this project.
        - contents:
            pasta_man.exceptions.RestrictedActivity (class)

Hierarchy:
    - encryption.py:
        - contents:
            - Encryption (class)
                - contents:
                    __init__
                    lock
                    unlock

Encryption class:
    - description: Encryption class is made for master key encryption.

__init__:
    - description: This method is called as soon as the Encryption class is initialized.
    - params:
        - masterp:
            - description: A byte type data to be encrypted or decrypted.
            - type: bytes

lock:
    - description: locks the given data.

unlock:
    - description: unlocks the given data.

Working:
    - When the class is initialized, only one operation out of lock or unlock can be performed.
    - To perform multiple tasks, the class needs to be initialized again.
"""

# import all required internal modules
from base64 import urlsafe_b64encode

# import all required external modules
from cryptography.fernet import Fernet
from cryptography.hazmat.primitives.hashes import SHA256
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

# import all project specific modules
from pasta_man.utilities.Exceptions.exceptions import RestrictedActivity

# Encryption Class Definition
class Encryption:
    # __init__ function
    def __init__(self, masterp: bytes):
        # define hidden variable
        self.__status = True
        # define a password (default currently)
        password = "pastaman".encode('ascii')
        
        # define a kdf for deriving a password from the above string.
        self._kdf = PBKDF2HMAC(
            algorithm=SHA256(), # algorithm used is SHA256
            length=32,
            iterations=480000,
            salt="pastaman".encode('ascii') # salt is an extra text added to improve protection.
        )
        
        # define class variable so that the masterp can be accessed all through the class
        self.str = masterp
        
        # derive the password using the kdf and encode it using urlsafe_b64encode
        _p = urlsafe_b64encode(self._kdf.derive(password))
        
        # define class fernet object.
        self._fernet = Fernet(_p)
    
    # lock function
    def lock(self) -> bytes:
        """## lock the masterp

        ### Raises:
            - `RestrictedActivity`: the class can be only used once.

        ### Returns:
            - `bytes`: returns locked bytes
        """
        # return locked bytes if status is True.
        if self.__status:
            try:
                return self._fernet.encrypt(self.str)
            finally:
                # after return, change status to False.
                self.__status = False
        # if status is false, raise error.
        else:
            raise RestrictedActivity(RestrictedActivity.RESTRICTEDACTIVITY, 'lock')
    
    # unlock function
    def unlock(self) -> bytes:
        """## unlock the string

        ### Raises:
            - `RestrictedActivity`: the class can be only used once

        ### Returns:
            - `bytes`: return unlocked bytes
        """
        # return unlocked bytes if status is True
        if self.__status:
            try:
                return self._fernet.decrypt(self.str)
            finally:
                # after return, change status ti False.
                self.__status = False
        # if status is False, raise error.
        else:
            raise RestrictedActivity(RestrictedActivity.RESTRICTEDACTIVITY, 'unlock')


#
# This file is responsible for all terminal related activities
#

from pasta_man.utilities.Exceptions.TerminalExceptions import ArgumentError, EmptyArgument
from pasta_man.architectures.targets import targets
from pasta_man.utilities.encryption import Encryption
from pasta_man.utilities.helptext import Helptext
from pasta_man.utilities.pasta_docs import Docstring

from os.path import dirname, abspath, isdir, join as jPath, splitext, isfile
from os import system as run, getcwd as pwd
from pathlib import Path
from optioner import options
from colorama import Fore as f
from shutil import rmtree
from wrapper_bar.wrapper import Wrapper
import sys, platform, threading, pyperclip
from getpass import getpass

class Terminal:
    def __init__(self, __version__:str, globalArgs: list[str], masterpassword: bytes) -> None:
        self.__masterpassword = masterpassword
        self.__argv = globalArgs
        self.__shortargs = ['h', 'v', 'p', 'rmc', 'dwl', 'i', 'e', 's']
        self.__longargs = ['help', 'version', 'path', 'remove-configurations', 'doc-w-list', 'import', 'export', 'search']
        self.__mutex = [
            ['help', 'h'],self.__remFromOriginal(['help', 'h']),
            ['v', 'version'], self.__remFromOriginal(['v', 'version']),
            ['p', 'path'], self.__remFromOriginal(['p', 'path']),
            ['rmc', 'remove-configurations'], self.__remFromOriginal(['rmc', 'remove-configurations']),
            ['dwl', 'doc-w-list'], self.__remFromOriginal(['dwl', 'doc-w-list']),
            ['i', 'import'], self.__remFromOriginal(['i', 'import']),
            ['e', 'export'], self.__remFromOriginal(['export', 'e']),
            ['s', 'search'], self.__remFromOriginal(['s', 'search'])
        ]

        self.__analyze()
        self.__help(__version__)
        self.__readOnlyArgs(__version__)
        self.__permanentEffectArgs()
        
    def __remFromOriginal(self, excpt: list[str]) -> list[str]:
        self.ORIGINAL = self.__shortargs + self.__longargs
        withoutExcept: list[str] = []
        for x in self.ORIGINAL:
            if x in excpt:
                continue
            else:
                withoutExcept.append(x)
        
        return withoutExcept
    
    def __analyze(self) -> None:
        if len(self.__argv) == 0:
            raise EmptyArgument(EmptyArgument.EMPTYARGUMENT)
        
        self.TerminalControl = options(self.__shortargs, self.__longargs, self.__argv, ifthisthennotthat=self.__mutex)
        self.arguments, self.argumentCheck, self.argumentError, self.argumentFalseArgs = self.TerminalControl._argparse()
        
        try:
            if not self.argumentCheck:
                raise ArgumentError(ArgumentError.ARGUMENTERROR, self.argumentError)
        except ArgumentError as e:
            print(e)
            
    
    def __help(self, __version__:str) -> None:
        """If help is asked, print help and exit"""
        if '-h' in self.arguments or '--help' in self.arguments:
            __text = Helptext(__version__)
            __text.helper()
        elif '-v' in self.arguments or '--version' in self.arguments:
            __text = Helptext(__version__)
            __text.showver()
        else:
            pass
    
    def __readOnlyArgs(self, __version__: str) -> None:
        """Arguments that perform readonly tasks"""
        if '-p' in self.arguments or '--path' in self.arguments:
            print(dirname(dirname(abspath(__file__)))) # pasta-man directory
            
        elif '-dwl' in self.arguments or '--doc-w-list' in self.arguments:
            docs = Docstring()
            print(f'{f.BLUE}Pasta Man{f.RESET}', f'{f.RED}v{__version__}{f.RESET}')
            print(f'{f.GREEN}Hierarchy:{f.RESET}')
            docs.listRecurseF()
            print('\nExample: pasta_man.architectures.gui')
            userin = input(f'{f.LIGHTGREEN_EX}pasta-man>{f.RESET} ')
            document = docs.fetch(userin)
            if document and platform.system()=='Windows':
                run("cls")
                print('\n'+f'{f.YELLOW}{userin}{f.RESET}', 'docstring: ')
                print(document)
            elif document:
                run("clear")
                print('\n'+f'{f.YELLOW}{userin}{f.RESET}', 'docstring: ')
                print(document)
            else:
                print(f"{f.RED}No docstring defined for {userin}{f.RESET}")
            
            
        elif '-e' in self.arguments or '--export' in self.arguments:
            try:
                from tkinter import filedialog
                path = filedialog.askdirectory(initialdir=pwd(), title="Export To")
            except ImportError:
                path = abspath(input("Export directory: "))
                while(not isdir(path)):
                    print(f"Not a Directory: {path}. Try Again!")
                    path = abspath(input("Export directory: "))
            
            ext = self.TerminalControl._what_is_('e')
            
            if path:
                target = targets(self.__masterpassword)
                t1 = threading.Thread(target=target.init)
                t1.start()
                t1.join()
                threading.Thread(target=target.export, args=(ext, path)).start()
            
            
        elif '-s' in self.arguments or '--search' in self.arguments:
            if type(self.TerminalControl._what_is_('s', 2)):
                keywordtype, keyword = self.TerminalControl._what_is_('s', 2)
            else:
                print("Error: \'-s\ and \'--search\' arguments require two values.'")
                sys.exit(1)
            
            codes = [
                f"""
target = targets(\'{self.__masterpassword.decode('ascii')}\'.encode('ascii'))
t1 = threading.Thread(target=target.init)
t1.start()
t1.join()
target.searchAll(\'{keyword}\', \'{keywordtype}\')
results = target.searchAllResults
results = pd.DataFrame(results)
"""
            ]
            
            dependencies = [
                """from pasta_man.architectures.targets import targets""",
                """import threading""",
                """import pandas as pd"""
            ]
            
            wrap = Wrapper()
            wrap.pyShellWrapper(codes, dependencies, delay=0.001, label="searching:")
            results = wrap.pyShellWrapperResults['results']
            if not results.empty:
                print(results.drop(['password', 'timestamp'], axis=1))
            else:
                print(f"{f.RED}No Match Found{f.RESET}")
                
            index = int(input(f"{f.BLUE}index>{f.RESET} "))
            results = results.iloc[index]
            
            # ask for master pass
            allegedpass = getpass(f"{f.RED}master-password>{f.RESET} ")
            
            # get master password
            with open(jPath(str(Path.home()), '.pastaman', '.m'), 'rb') as m:
                masterpassword = m.read() # this is encrypted.
            
            thread = threading.Thread(target=self.decryptthread, args=(masterpassword.decode('ascii'),))
            thread.start()
            thread.join()
            
            if allegedpass == self.den.decode('ascii'):
                codes = [
                    f"""
target = targets(\'{self.__masterpassword.decode('ascii')}\'.encode('ascii'))
t1 = threading.Thread(target=target.init)
t1.start()
t1.join()
thread = threading.Thread(target=target.decrypt, args=(\'{results['password']}\'.encode('ascii'),))
thread.start()
thread.join()
pyperclip.copy(target.dec)
spam = pyperclip.paste()
"""
                ]
                
                dependencies = [
                    """import threading""",
                    """import pyperclip""",
                    """from pasta_man.architectures.targets import targets"""
                ]
                
                wrap.pyShellWrapper(codes, dependencies, delay=0.001, label="decrypting:")
                print(f"{f.GREEN}Password copied to clipboard.{f.RESET}")
            
            
        else:
            pass
    
    def decryptthread(self, masterpassword: str):
        denc = Encryption(masterpassword.encode('ascii'))
        self.den = denc.unlock()
        
    
    def __permanentEffectArgs(self):
        if '-rmc' in self.arguments or '--remove-configurations' in self.arguments:
            rmtree(jPath(str(Path.home()), '.pastaman'))
            
        elif '-i' in self.arguments or '--import' in self.arguments:
            try:
                from tkinter import filedialog
                path = filedialog.askopenfilename(title="Import", initialdir=pwd(), filetypes=[("Excel Files", "*.xlsx"), ("csv Files", "*.csv")], defaultextension="*.csv")
            except ImportError:
                path = abspath(input("Import: "))
                while(not isfile(path)):
                    print(f"Not a File. Try Again!")
                    path = abspath(input("Import: "))
                
            if path:
                p, ext = splitext(path)
                target = targets(masterpassword=self.__masterpassword)
                t1 = threading.Thread(target=target.init)
                t1.start()
                t1.join()
                threading.Thread(target=target.importer, args=(ext, path)).start()
            
            
        else:
            pass


"""
Modules:
    - internal:
        - description: Already Comes installed with your python interpreter.
        - contents:
            sys (module)
    - external:
        - description: Needs to be installed using pip.
        -contents:
            colorama.Fore (variable)
    - Project Specific
        - description: Modules made for this project.

Hierarchy:
    - helptext.py
        - contents:
            - helptext (class):
                - contents:
                    __init__
                    helper
                    showver

helptext class:
    - description: contains all the predefined helptexts

__init__:
    - description: this is called as soon as helptext class is initialized

helper:
    - description: this shows help for entire pasta-man

showver:
    - description: this shows version info of pasta-man

Working:
    - show help whenever asked.
"""


from colorama import Fore as f
import sys

class Helptext:
    def __init__(self, __version__: str):
        self.__version__ = __version__
    
    def helper(self):
        print(f'{f.BLUE}Pasta Man{f.RESET}', f'{f.RED}v{self.__version__}{f.RESET}')
        print("helptext")
        print("  |  -h or --help                     : show this help and exit.")
        print("  |  -v or --version                  : show version and exit.")
        print("  |  -p or --path                     : show install path and exit.")
        print("  |  -rmc or --remove-configurations  : remove existing configs. ["+f'{f.RED}Warning{f.RESET}'+"] This is irreversible.")
        print("  |  -dwl or --doc-w-list             : list all modules of pasta-man. Enter the full-module-name for docstring.")
        print("  |  -i or --import                   : import a passwords file. Only files exported by pasta-man can be imported.")
        print(f"  |                                     {f.RED}Syntax:{f.RESET} pasta-man --import")
        print("  |  -e or --export                   : export passwords.")
        print(f"  |                                     {f.RED}Syntax:{f.RESET} pasta-man --export <export-format>")
        print("  |                                     Available export formats -> [\'csv\', \'xlsx\']")
        print("  |  -s or --search                   : search a keyword in keyword-type.")
        print(f"  |                                     {f.RED}Syntax:{f.RESET} pasta-man --search <keyword-type> <keyword>")
        print("  |                                     Available keyword types -> [\'target\', \'target-type\', \'username\']")
    
    def showver(self):
        print(f'{f.BLUE}Pasta Man{f.RESET}', f'{f.RED}v{self.__version__}{f.RESET}')
        print('author: d33pster', 'GitHub:', f'{f.LIGHTBLUE_EX}https://github.com/d33pster/pasta-man{f.RESET}')


"""
Modules:
    - internal:
        - description: Already Comes installed with your python interpreter.
        - contents:
            pathlib.Path (class)
            os.scandir (function)
            os.path.dirname (function)
            os.path.abspath (function)
            inspect
    - external:
        - description: Needs to be installed using pip.
    - Project Specific
        - description: Modules made for this project.

Hierarchy:
    - pasta_docs.py
        - contents:
            - docstring (class)
                -contents:
                    __init__
                    list_recursive
                    listRecurseF
                    fetch

docstring class:
    - description: docstring class is made to fetch the docstring of a file just for pasta-man.

__init__:
    - description: This method is called as soon as the docstring class is initialized.

list_recursive:
    - description: This method can return a list of all complete module paths of pasta-man.
    - return:
        - type: list[str]

listRecurseF:
    - description: print level wise hierarchy of modules of pasta-man

fetch:
    - description: fetch the main docstring from the module file.
    - params:
        - module:
            - description: the module whose docstring needs to be fetched.
            - type: str

working:
    - This file is responsible for resolving and fetching docstrings defined inside the modules.
"""

# import internal modules
from os.path import dirname, abspath
from os import scandir
from pathlib import Path
import inspect

# class definition
class Docstring:
    # __init__ function
    def __init__(self):
        """## Initialise the docstring class
        """
        self.parentmodule = "pasta_man"
        self.directory = dirname(dirname(abspath(__file__))) # inside pasta_man dir
        # print(self.directory)
    
    def __fetch(self, submod: list[str] = [], daddy:bool = False) -> str | None:
        """## fetch the docstring of the given submodule of pasta-man

        ### Args:
            - `submod (list[str], optional)`: submodule of pasta-man. Defaults to [].
            - `daddy (bool, optional)`: Set this to True if you want to fetch docstring of parent module -> pasta-man. Defaults to False.

        ### Returns:
            - `str | None`: Returns docstring if defined, else None.
        """
        # if parent module is queried.
        if daddy or len(submod)==0:
            try:
                module = __import__(self.parentmodule)
                return inspect.getdoc(module)
            except ImportError:
                return None
        # if a submod is provided.
        else:
            # check the submod if there is more values to it.
            if len(submod)>1:
                submodules = submod
                submodlast = submodules[len(submodules)-1] # last element
                submodulesfirsts = submodules[:len(submodules)-1] # all other element in order.
                
                # merge the first elements to daddy
                dadmod = self.parentmodule
                for x in submodulesfirsts:
                    dadmod += f".{x}"
                
                # find docstring if any
                try:
                    module = __import__(dadmod, fromlist=[submodlast])
                    submodule = getattr(module, submodlast)
                    return inspect.getdoc(submodule)
                except (ImportError, AttributeError):
                    return None
            elif len(submod)==1:
                try:
                    module = __import__(self.parentmodule, fromlist=[submod[0]])
                    submodule = getattr(module, submod[0])
                    return inspect.getdoc(submodule)
                except (ImportError, AttributeError):
                    return None
    
    def __scan_recursive(self, directory: str):
        """## recursively scan a directory to list all paths

        ### Args:
            - `directory (str)`: directory to be scanned.

        ### Yields:
            - `Generator[DirEntry[str] | Any, Any, None]`
        """
        for entry in scandir(directory):
            if entry.is_file():
                yield entry
            else:
                yield from self.__scan_recursive(entry.path)
    
    def list_recursive(self) -> list[str]:
        """## list only pasta-man modules.

        ### Returns:
            - `list[str]`: list of pasta-man modules.
        """
        modules = []
        for item in self.__scan_recursive(self.directory):
            path = str(Path(item))[1:].replace('.py', '').replace('/','.').replace('\\','.')
            if '__pycache__' in path:
                continue
            
            path = self.parentmodule+path.split(self.parentmodule)[1]
            if path==self.parentmodule+".":
                path = self.parentmodule+"."+self.parentmodule
            modules.append(path)
        
        return modules
    
    def __construct_hierarchy(self, strings: list[str]) -> dict:
        """## Construct a hierarchy of all the modules from the list

        ### Args:
            - `strings (list[str])`: list of modules

        ### Returns:
            - `dict`: hierarchy dict
        """
        hierarchy = {}
        for string in strings:
            parts = string.split(".")
            current_level = hierarchy
            for part in parts:
                if part not in current_level:
                    current_level[part] = {}
                current_level = current_level[part]
        return hierarchy

    def __print_hierarchy(self, hierarchy:dict, indent:str="") -> None:
        """## print hierarchy from hierarchy dict

        ### Args:
            - `hierarchy (dict)`: hierarchy dict
            - `indent (str, optional)`: indent value. Defaults to "".
        """
        for key, value in hierarchy.items():
            print(indent + "->" + key)
            if value:
                self.__print_hierarchy(value, indent + "   ")

    def listRecurseF(self) -> None:
        """## print hierarchy of pasta-man modules
        """
        modules = self.list_recursive()
        self.__print_hierarchy(self.__construct_hierarchy(modules))
    
    def fetch(self, module:str) -> str | None:
        """## fetch the docstring of pasta-man module if any.

        ### Args:
            - `module (str)`: module path

        ### Returns:
            - `str | None`: returns docstring or None
        """
        if len(module.split('.'))==1:
            return self.__fetch(daddy=True)
        elif len(module.split('.'))>1:
            return self.__fetch(submod=module.split('.')[1:])
        else:
            return None


#
# This file is responsible for retransforming imported passwords
#

from cryptography.fernet import Fernet
from cryptography.hazmat.primitives.hashes import SHA256
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from base64 import urlsafe_b64encode
from os.path import join as jPath
from datetime import datetime
from pathlib import Path
import pandas as pd

#
# Format of the imported file will be -> 1. masterrow, 2. initrow, 3. data ...
#
class Retransform:
    def __init__(self, datapath: str, ext: str = "csv"):
        if ext==".csv":
            self.__data = pd.read_csv(datapath, header=None, names=['target-type', 'target', 'username', 'password'])
        elif ext==".xlsx":
            self.__data = pd.read_excel(datapath, header=None, names=['target-type', 'target', 'username', 'password'])
        self.__masterrow = self.__data.iloc[0]
        self.__data = self.__data.drop(0) # drop master column,
        self.__data = self.__data.drop(1) # drop init column.
        print(self.__data)
        self.__oldEncryptedMasterPassword:str = self.__masterrow['password']
        __kdf = PBKDF2HMAC(
            algorithm=SHA256(),
            length=32,
            salt="pastaman".encode('ascii'),
            iterations=480000,
        )
        __password = urlsafe_b64encode(__kdf.derive('pastaman'.encode('ascii')))
        __fernet = Fernet(__password)
        self.__oldDecryptedMasterPassword:bytes = __fernet.decrypt(self.__oldEncryptedMasterPassword.encode('ascii'))
        
        __oldkdf = PBKDF2HMAC(
            algorithm=SHA256(),
            length=32,
            salt="pastaman".encode('ascii'),
            iterations=480000,
        )
        __oldpassword = urlsafe_b64encode(__oldkdf.derive(self.__oldDecryptedMasterPassword))
        self.__oldfernet = Fernet(__oldpassword)
        
        __newkdf = PBKDF2HMAC(
            algorithm=SHA256(),
            length=32,
            salt="pastaman".encode('ascii'),
            iterations=480000,
        )
        
        with open(jPath(str(Path.home()), '.pastaman', '.m'), 'rb') as mfile:
            __newpassword = urlsafe_b64encode(__newkdf.derive(mfile.read()))
        
        self.__newfernet = Fernet(__newpassword)
        
        self.__newdata:list[dict] = []
        for i in range(len(self.__data)):
            row = self.__data.iloc[i]
            element = {
                "target-type":row['target-type'],
                "target":row['target'],
                "username":row['username'],
                "password":self.__newfernet.encrypt(self.__oldfernet.decrypt(row['password'].encode('ascii'))).decode('ascii'),
                "timestamp":datetime.today().strftime("%Y-%m-%d")
            }
            self.__newdata.append(element)
    
    def __fetchdata(self) -> list[dict]:
        return self.__newdata

    data = property(fget=__fetchdata)